"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_generator_1 = require("./types.generator");
var utilities_1 = require("../utilities");
var fragments_generator_1 = require("./fragments.generator");
var query_generator_1 = require("./query.generator");
var hooks_generator_1 = require("./hooks.generator");
exports.createMethodsArgs = function (field, prefix, suffix) {
    var _a = field.args.reduce(function (acc, arg) {
        var argName = arg.name;
        var type = types_generator_1.generateQGLArg(arg);
        acc.GQLVariables.push("$" + argName + ": " + type);
        acc.GQLArgs.push(argName + ": $" + argName);
        return acc;
    }, {
        GQLVariables: [],
        GQLArgs: [],
    }), GQLVariables = _a.GQLVariables, GQLArgs = _a.GQLArgs;
    var methodArgsType = '';
    if (field.args.length) {
        var parsedSuffix = 'Args' + (suffix ? suffix : '');
        methodArgsType = "" + (prefix ? prefix : '') + field.name + parsedSuffix;
    }
    return {
        GQLArgs: GQLArgs,
        GQLVariables: GQLVariables,
        methodArgsType: methodArgsType,
    };
};
exports.createGraphQLFunction = function (_a) {
    var field = _a.field, ObjectTypes = _a.ObjectTypes, prefix = _a.prefix, suffix = _a.suffix, type = _a.type, renderedFragmentInner = _a.renderedFragmentInner;
    var hasArgs = field.args.length > 0;
    var methodName = field.name;
    var methodArgsType = exports.createMethodsArgs(field, prefix, suffix).methodArgsType;
    var isScalar = utilities_1.evaluateType(field).isScalar;
    var returnedTypeDisplay = types_generator_1.getOneTSTypeDisplay({ field: field, prefix: prefix, suffix: suffix });
    var Query = query_generator_1.queryBuilder({ field: field, isScalar: isScalar, prefix: prefix, suffix: suffix, renderedFragmentInner: renderedFragmentInner, type: type });
    var withArgs = hasArgs
        ? utilities_1.areAllArgsOptional(field.args)
            ? 'WithOptionalArgs'
            : 'WithArgs'
        : '';
    if (isScalar) {
        return "\n    " + (field.description ? "/** " + field.description + " */" : '') + "\n    " + methodName + "(): Abortable" + type.high + withArgs + "<" + returnedTypeDisplay + (hasArgs ? ',' + methodArgsType : '') + "> {\n      const " + type.little + " = " + Query + "\n    return abortable" + type.high + "(" + type.little + ", " + hasArgs + ");\n        }\n    ,";
    }
    else {
        return "\n    " + (field.description ? "/** " + field.description + " */" : '') + "\n    " + methodName + "(): Fragmentable" + type.high + withArgs + "<" + returnedTypeDisplay + (hasArgs ? ',' + methodArgsType : '') + "> {\n    return {\n      $fragment: (fragment: string | DocumentNode) => {\n        const { isString, isFragment, fragmentName } = guessFragmentType(fragment);\n        const " + type.little + " = " + Query + "\n\n        return abortable" + type.high + "(" + type.little + ", " + hasArgs + ");\n      }\n    }\n      }\n  ,";
    }
};
exports.buildMethod = function (_a) {
    var field = _a.field, type = _a.type, ObjectTypes = _a.ObjectTypes, mode = _a.mode, prefix = _a.prefix, suffix = _a.suffix;
    var _b;
    var _c = utilities_1.evaluateType(field), isScalar = _c.isScalar, isEnum = _c.isEnum, typeName = _c.typeName;
    var fragmentDisplay = "${isString ? fragment : '...' + fragmentName}";
    var renderedFragmentInner = fragmentDisplay;
    if (!isScalar && !isEnum && utilities_1.isReturnTypeEdge(ObjectTypes, typeName)) {
        renderedFragmentInner = (_b = fragments_generator_1.createConnectionFragment(typeName, ObjectTypes, fragmentDisplay), (_b !== null && _b !== void 0 ? _b : fragmentDisplay));
    }
    if (mode === 'template') {
        return query_generator_1.createQueryFunction({ field: field, prefix: prefix, suffix: suffix, type: type, renderedFragmentInner: renderedFragmentInner });
    }
    else if (mode === 'hooks') {
        return hooks_generator_1.createApolloHook({
            ObjectTypes: ObjectTypes,
            field: field,
            prefix: prefix,
            suffix: suffix,
            type: type,
            renderedFragmentInner: renderedFragmentInner,
        });
    }
    else {
        return exports.createGraphQLFunction({
            ObjectTypes: ObjectTypes,
            field: field,
            prefix: prefix,
            suffix: suffix,
            type: type,
            renderedFragmentInner: renderedFragmentInner,
        });
    }
};
