"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var utilities_1 = require("../utilities");
var scalars_store_1 = __importDefault(require("../store/scalars.store"));
exports.generateEnumType = function (object, prefix, suffix) {
    var ObjectName = object.name;
    var generatedFields = object.enumValues.map(function (field) { return field.name; });
    return "\n  " + (object.description ? "/** " + object.description + " */" : '') + " export enum " + (prefix ? prefix : '') + ObjectName + (suffix ? suffix : '') + " {\n        " + generatedFields.map(function (enumType) { return utilities_1.capitalize(enumType) + " = \"" + enumType + "\""; }).join(',\n') + "\n  }";
};
exports.getOneTSTypeDisplay = function (_a) {
    var field = _a.field, prefix = _a.prefix, suffix = _a.suffix;
    var _b = utilities_1.evaluateType(field), isScalar = _b.isScalar, typeName = _b.typeName, isArray = _b.isArray;
    var returnedName = isScalar
        ? scalars_store_1.default.listScalars[typeName]
        : "" + (prefix ? prefix : '') + typeName + (suffix ? suffix : '');
    return returnedName + (isArray ? '[]' : '');
};
exports.generatedTsFields = function (fields, prefix, suffix) {
    return fields.map(function (field) {
        var propertyName = field.name;
        var isOptional = utilities_1.evaluateType(field).isOptional;
        var TStypeName = exports.getOneTSTypeDisplay({ field: field, prefix: prefix, suffix: suffix });
        return (field.description ? "/** " + field.description + " */" : '') + "\n    " + propertyName + (isOptional ? '?' : '') + ": " + (isOptional ? "Maybe<" + TStypeName + ">" : TStypeName) + ";";
    });
};
exports.generateQGLArg = function (field) {
    var _a = utilities_1.evaluateType(field), isArray = _a.isArray, isArrayOptional = _a.isArrayOptional, isOptional = _a.isOptional, typeName = _a.typeName;
    return "" + (isArray ? '[' : '') + typeName + (isOptional ? '' : '!') + (isArray ? ']' : '') + (isArrayOptional ? '!' : '');
};
exports.getObjectTSInterfaces = function (object, prefix, suffix) {
    var fieldsKey = object.kind === 'OBJECT' || object.kind === 'INTERFACE' ? 'fields' : 'inputFields';
    var generatedFields = exports.generatedTsFields(object[fieldsKey], prefix, suffix);
    return exports.buildTsInterfaceString(object, generatedFields, prefix, suffix);
};
exports.getQueriesArgsTSInterfaces = function (object, prefix, suffix) {
    var ObjectName = object.name;
    var parsedSuffix = 'Args' + (suffix ? suffix : '');
    var generatedFields = exports.generatedTsFields(object.args, prefix, suffix);
    return exports.buildTsInterfaceString(object, generatedFields, prefix, parsedSuffix);
};
exports.buildTsInterfaceString = function (type, fields, prefix, suffix) {
    return (type.description ? "/** " + type.description + " */" : '') + "\n  export interface " + (prefix ? prefix : '') + type.name + (suffix ? suffix : '') + " {\n    " + fields.join('\n') + "\n  }\n";
};
